--- old/usr/src/tools/smatch/src/smatch_mtag_data.c	Mon Aug  5 08:38:38 2019
+++ new/usr/src/tools/smatch/src/smatch_mtag_data.c	Mon Aug  5 08:38:38 2019
@@ -36,11 +36,9 @@
 	return 0;
 }
 
-static struct range_list *select_orig_rl(sval_t sval)
+static struct range_list *select_orig(mtag_t tag, int offset)
 {
 	struct range_list *rl = NULL;
-	mtag_t tag = sval.uvalue & ~MTAG_OFFSET_MASK;
-	int offset = sval.uvalue & MTAG_OFFSET_MASK;
 
 	mem_sql(&save_rl, &rl, "select value from mtag_data where tag = %lld and offset = %d;",
 		tag, offset);
@@ -71,11 +69,8 @@
 	return 0;
 }
 
-void insert_mtag_data(sval_t sval, struct range_list *rl)
+static void insert_mtag_data(mtag_t tag, int offset, struct range_list *rl)
 {
-	mtag_t tag = sval.uvalue & ~MTAG_OFFSET_MASK;
-	int offset = sval.uvalue & MTAG_OFFSET_MASK;
-
 	rl = clone_rl_permanent(rl);
 
 	mem_sql(NULL, NULL, "delete from mtag_data where tag = %lld and offset = %d and type = %d",
@@ -87,8 +82,10 @@
 void update_mtag_data(struct expression *expr)
 {
 	struct range_list *orig, *new, *rl;
+	struct symbol *type;
 	char *name;
-	sval_t sval;
+	mtag_t tag;
+	int offset;
 
 	name = expr_to_var(expr);
 	if (is_kernel_param(name)) {
@@ -97,20 +94,27 @@
 	}
 	free_string(name);
 
-	if (!get_mtag_addr_sval(expr, &sval))
+	if (!expr_to_mtag_offset(expr, &tag, &offset))
 		return;
 
+	type = get_type(expr);
+	if ((offset == 0) &&
+	    (!type || type == &void_ctype ||
+	     type->type == SYM_STRUCT || type->type == SYM_UNION || type->type == SYM_ARRAY))
+		return;
+
 	get_absolute_rl(expr, &rl);
 
-	orig = select_orig_rl(sval);
+	orig = select_orig(tag, offset);
 	new = rl_union(orig, rl);
-	insert_mtag_data(sval, new);
+	insert_mtag_data(tag, offset, new);
 }
 
 static void match_global_assign(struct expression *expr)
 {
 	struct range_list *rl;
-	sval_t sval;
+	mtag_t tag;
+	int offset;
 	char *name;
 
 	name = expr_to_var(expr->left);
@@ -120,11 +124,11 @@
 	}
 	free_string(name);
 
-	if (!get_mtag_addr_sval(expr->left, &sval))
+	if (!expr_to_mtag_offset(expr->left, &tag, &offset))
 		return;
 
 	get_absolute_rl(expr->right, &rl);
-	insert_mtag_data(sval, rl);
+	insert_mtag_data(tag, offset, rl);
 }
 
 static int save_mtag_data(void *_unused, int argc, char **argv, char **azColName)
@@ -171,22 +175,25 @@
 }
 
 struct db_cache_results {
-	sval_t sval;
+	mtag_t tag;
 	struct range_list *rl;
 };
 static struct db_cache_results cached_results[8];
 
-static int get_rl_from_mtag_sval(sval_t sval, struct symbol *type, struct range_list **rl)
+static int get_rl_from_mtag_offset(mtag_t tag, int offset, struct symbol *type, struct range_list **rl)
 {
 	struct db_info db_info = {};
-	mtag_t tag;
-	int offset;
+	mtag_t merged = tag | offset;
 	static int idx;
 	int ret;
 	int i;
 
+	if (!type || type == &void_ctype ||
+	    (type->type == SYM_STRUCT || type->type == SYM_ARRAY || type->type == SYM_UNION))
+		return 0;
+
 	for (i = 0; i < ARRAY_SIZE(cached_results); i++) {
-		if (sval.uvalue == cached_results[i].sval.uvalue) {
+		if (merged == cached_results[i].tag) {
 			if (cached_results[i].rl) {
 				*rl = cached_results[i].rl;
 				return 1;
@@ -195,12 +202,6 @@
 		}
 	}
 
-	tag = sval.uvalue & ~MTAG_OFFSET_MASK;
-	offset = sval.uvalue & MTAG_OFFSET_MASK;
-	if (offset == MTAG_OFFSET_MASK) {
-		ret = 0;
-		goto update_cache;
-	}
 	db_info.type = type;
 
 	run_sql(get_vals, &db_info,
@@ -216,7 +217,7 @@
 	ret = 1;
 
 update_cache:
-	cached_results[idx].sval = sval;
+	cached_results[idx].tag = merged;
 	cached_results[idx].rl = db_info.rl;
 	idx = (idx + 1) % ARRAY_SIZE(cached_results);
 
@@ -231,16 +232,19 @@
 int get_mtag_rl(struct expression *expr, struct range_list **rl)
 {
 	struct symbol *type;
-	sval_t sval;
+	mtag_t tag;
+	int offset;
 
-	if (!get_mtag_addr_sval(expr, &sval))
+	if (!expr_to_mtag_offset(expr, &tag, &offset))
 		return 0;
+	if (offset >= MTAG_OFFSET_MASK)
+		return 0;
 
 	type = get_type(expr);
 	if (!type)
 		return 0;
 
-	return get_rl_from_mtag_sval(sval, type, rl);
+	return get_rl_from_mtag_offset(tag, offset, type, rl);
 }
 
 void register_mtag_data(int id)
